SystemOrganization addCategory: #ExternalWebBrowser!

Object subclass: #ExternalWebBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

ExternalWebBrowser class
	instanceVariableNames: 'specificBrowserClass'!

ExternalWebBrowser class
	instanceVariableNames: 'specificBrowserClass'!

!ExternalWebBrowser class methodsFor: 'class initialization' stamp: 'laza 11/27/2004 00:14'!
initialize
	"ExternalWebBrowser initialize"
	Smalltalk addToStartUpList: self.
	WebBrowser register: self.
	self startUp.! !

!ExternalWebBrowser class methodsFor: 'instance creation' stamp: 'noha 8/5/2007 13:29'!
new
	^ specificBrowserClass new! !

!ExternalWebBrowser class methodsFor: 'scamper interface' stamp: 'laza 11/27/2004 00:14'!
newOrExistingOn: aStringOrUrl
	self new open: aStringOrUrl asUrl! !

!ExternalWebBrowser class methodsFor: 'scamper interface' stamp: 'laza 11/27/2004 00:14'!
openOnUrl: aStringOrUrl
	self new open: aStringOrUrl asUrl! !

!ExternalWebBrowser class methodsFor: 'system startup' stamp: 'noha 8/5/2007 14:00'!
startUp
	| browsers |
	browsers := (ExternalWebBrowserPlatform allSubclasses select: [
		:each | 
		each subclasses size isZero and: [
			each suitsPlatform
		]
	]) asSortedCollection: [
		:a :b | 
		a precedence < b precedence
	].
	specificBrowserClass := browsers first.! !

Object subclass: #ExternalWebBrowserPlatform
	instanceVariableNames: 'currentUrl'
	classVariableNames: 'StartUrl'
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

ExternalWebBrowserPlatform class
	instanceVariableNames: 'specificBrowserClass'!

ExternalWebBrowserPlatform class
	instanceVariableNames: 'specificBrowserClass'!

ExternalWebBrowserPlatform subclass: #ExternalWebBrowserMacOS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserMacOS commentStamp: 'dd 11/5/2004 12:56' prior: 0!
This class uses Applescript to open an url with the default web browser. The inner work is very simple: it verifies that the Applescript package (see SqueakMap) is available and sends the 'open location' command to the system.

If you have installed the ExternalWebBrowser package before installing the Applescript package, you will have to execute this command:

	ExternalWebBrowser initialize!

!ExternalWebBrowserMacOS class methodsFor: 'testing' stamp: 'laza 12/9/2004 08:51'!
isApplescriptAvailable
	| package |
	^(Smalltalk hasClassNamed: #Applescript)	
		or: [(package := (SMSqueakMap default loadUpdates) packageWithName: 'Applescript') notNil
			and: [package isInstallableAndNotInstalled
				and: [([package install isInstalled] on: Error do: [false])
					and: [Smalltalk hasClassNamed: #Applescript]]]]
! !

!ExternalWebBrowserMacOS class methodsFor: 'testing' stamp: 'dd 12/9/2004 17:34'!
suitsPlatform
	^SmalltalkImage current platformName = 'Mac OS'
			and: [self isApplescriptAvailable]! !

!ExternalWebBrowserMacOS methodsFor: 'external calls' stamp: 'laza 12/9/2004 08:57'!
platformOpen: url
	Applescript doIt: 'open location "', url asText, '"'! !

!ExternalWebBrowserPlatform class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:54'!
precedence
	^ 1! !

!ExternalWebBrowserPlatform class methodsFor: 'accessing' stamp: 'laza 5/17/2004 07:36'!
startUrl: aStringOrUrl
	StartUrl := aStringOrUrl asString! !

!ExternalWebBrowserPlatform class methodsFor: 'testing' stamp: 'laza 5/17/2004 06:58'!
suitsPlatform
	^false! !

!ExternalWebBrowserPlatform methodsFor: 'scamper interface' stamp: 'laza 5/18/2004 13:25'!
jumpToAbsoluteUrl: aStringOrUrl
	self open: aStringOrUrl asUrl! !

!ExternalWebBrowserPlatform methodsFor: 'scamper interface' stamp: 'laza 5/17/2004 08:53'!
jumpToNewUrl
	"change to a new, user-specified page"
	| newUrl |
	newUrl := FillInTheBlank request: 'url to visit' translated initialAnswer: currentUrl asText.
	(newUrl isNil or: [ newUrl isEmpty ]) ifTrue: [ ^self ].
	self jumpToAbsoluteUrl: newUrl! !

!ExternalWebBrowserPlatform methodsFor: 'scamper interface' stamp: 'laza 5/18/2004 13:25'!
jumpToUrl: aStringOrUrl
	self open: aStringOrUrl asUrl! !

!ExternalWebBrowserPlatform methodsFor: 'external calls' stamp: 'dd 12/9/2004 16:16'!
open: url
	currentUrl := url.
	self platformOpen: url! !

!ExternalWebBrowserPlatform methodsFor: 'external calls' stamp: 'laza 12/9/2004 08:26'!
platformOpen: url
	self subclassResponsibility! !

ExternalWebBrowserPlatform subclass: #ExternalWebBrowserUnix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserUnix class methodsFor: 'constants' stamp: 'laza 5/19/2004 23:41'!
helperCommand
	^self subclassResponsibility! !

!ExternalWebBrowserUnix class methodsFor: 'constants' stamp: 'dd 12/9/2004 17:37'!
helperProgram
	^self subclassResponsibility! !

!ExternalWebBrowserUnix class methodsFor: 'testing' stamp: 'laza 12/9/2004 09:06'!
isCLibAvailable
	| moduleNames index |
	moduleNames := #('c' 'libc.so.6'). "standard, debian"
	index := 1.
	[^(self new system: nil) ~= 0] on: Error do: [:ex |
		index <= moduleNames size ifTrue: [
			((ExternalWebBrowserUnix compiledMethodAt: #system:) literalAt: 1)
				setModule: (moduleNames at: index).
			index := index + 1.
			ex retry].
		^false]! !

!ExternalWebBrowserUnix class methodsFor: 'testing' stamp: 'laza 11/27/2004 00:26'!
suitsPlatform
	^(SmalltalkImage current platformName = 'unix'
		and: [self isCLibAvailable])
		and: [(self new system: 'which ', self helperProgram) = 0]! !

!ExternalWebBrowserUnix methodsFor: 'external calls' stamp: 'laza 12/9/2004 08:27'!
platformOpen: url
	self system: self class helperProgram, ' ', (self class helperCommand format: {url asText})! !

!ExternalWebBrowserUnix methodsFor: 'as yet unclassified' stamp: 'laza 5/19/2004 23:34'!
system: string 
	"system() executes a command specified in string by calling /bin/sh -c 
	string, and returns after the command has been completed."
	<apicall: short 'system' (char*) module: 'c'>
	^self externalCallFailed! !

ExternalWebBrowserUnix subclass: #ExternalWebBrowserUnixGnome
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserUnixGnome class methodsFor: 'constants' stamp: 'noha 8/5/2007 12:02'!
helperCommand
	^' "{1}"'! !

!ExternalWebBrowserUnixGnome class methodsFor: 'constants' stamp: 'noha 8/5/2007 12:02'!
helperProgram
	^'gnome-open'! !

!ExternalWebBrowserUnixGnome class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:18'!
precedence
	^ 2! !

!ExternalWebBrowserUnixGnome class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:53'!
suitsPlatform
	"Gnome should be running to use me"
	^ super suitsPlatform and: [
		(self new system: 'test x''$GNOME_DESKTOP_SESSION_ID'' !!= x''''') = 0
	]! !

ExternalWebBrowserUnix subclass: #ExternalWebBrowserUnixKDE
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserUnixKDE class methodsFor: 'constants' stamp: 'laza 5/19/2004 23:45'!
helperCommand
	^'openURL "{1}"'! !

!ExternalWebBrowserUnixKDE class methodsFor: 'constants' stamp: 'laza 5/19/2004 23:57'!
helperProgram
	^'kfmclient'! !

!ExternalWebBrowserUnixKDE class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:17'!
precedence
	^ 2! !

!ExternalWebBrowserUnixKDE class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:50'!
suitsPlatform
	"KDE should be running to use me"
	^ super suitsPlatform and: [
		(self new system: 'test x''$KDE_FULL_SESSION'' = x''true''') = 0	
	]! !

ExternalWebBrowserUnix subclass: #ExternalWebBrowserUnixMacOSX
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserUnixMacOSX class methodsFor: 'constants' stamp: 'laza 12/9/2004 09:02'!
helperCommand
	^'''{1}'''! !

!ExternalWebBrowserUnixMacOSX class methodsFor: 'constants' stamp: 'laza 12/9/2004 09:02'!
helperProgram
	^'open'! !

!ExternalWebBrowserUnixMacOSX class methodsFor: 'as yet unclassified' stamp: 'noha 8/3/2007 13:35'!
suitsPlatform
	^ super suitsPlatform and: [
		(self new system: 'uname -a | grep ''^Darwin''') = 0
	]! !

ExternalWebBrowserUnix subclass: #ExternalWebBrowserUnixMozilla
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

ExternalWebBrowserUnixMozilla class
	instanceVariableNames: 'helperProgram'!

ExternalWebBrowserUnixMozilla class
	instanceVariableNames: 'helperProgram'!

!ExternalWebBrowserUnixMozilla class methodsFor: 'constants' stamp: 'noha 8/5/2007 13:44'!
helperCommand
	^' "{1}"'! !

!ExternalWebBrowserUnixMozilla class methodsFor: 'constants' stamp: 'noha 8/5/2007 13:44'!
helperProgram
	^ helperProgram! !

!ExternalWebBrowserUnixMozilla class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:17'!
precedence
	^ 3! !

!ExternalWebBrowserUnixMozilla class methodsFor: 'as yet unclassified' stamp: 'mtf 12/12/2007 15:33'!
suitsPlatform
	#( firefox mozilla ) do: [:browser |
		helperProgram := browser.
		super suitsPlatform ifTrue: [^ true]].
	^ false! !

ExternalWebBrowserUnix subclass: #ExternalWebBrowserUnixXdgUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserUnixXdgUtils commentStamp: 'noha 8/5/2007 13:57' prior: 0!
I can open a browser on a modern linux desktop. I use xdg-utils from the portland project. See

http://portland.freedesktop.org/wiki/!

!ExternalWebBrowserUnixXdgUtils class methodsFor: 'constants' stamp: 'noha 8/5/2007 12:15'!
helperCommand
	^' "{1}"'! !

!ExternalWebBrowserUnixXdgUtils class methodsFor: 'constants' stamp: 'noha 8/5/2007 12:15'!
helperProgram
	^'xdg-open'! !

ExternalWebBrowserPlatform subclass: #ExternalWebBrowserUnsupported
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserUnsupported class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:23'!
precedence
	^ 4! !

!ExternalWebBrowserUnsupported class methodsFor: 'as yet unclassified' stamp: 'noha 8/5/2007 13:24'!
suitsPlatform
	^ true! !

!ExternalWebBrowserUnsupported methodsFor: 'external calls' stamp: 'dd 12/9/2004 16:27'!
platformOpen: url
	self inform: 'Sorry!! I don''t know how to open an\external web browser on this platform!!' withCRs. 
	! !

ExternalWebBrowserPlatform subclass: #ExternalWebBrowserWindows
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExternalWebBrowser'!

!ExternalWebBrowserWindows class methodsFor: 'testing' stamp: 'laza 5/17/2004 07:04'!
suitsPlatform
	^SmalltalkImage current platformName = 'Win32'! !

!ExternalWebBrowserWindows methodsFor: 'external calls' stamp: 'laza 12/9/2004 08:27'!
platformOpen: url
	Win32Shell new shellOpen: url asString! !
ExternalWebBrowser initialize!
